/*
	Copyright (c) 1998-2001 Macromedia, Inc.	All Rights Reserved

	You may utilize this source file to create and compile object code for use within products
	you may create. THIS CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, AND
	MACROMEDIA DISCLAIMS ALL IMPLIED WARRANTIES INCLUDING, BUT NOT LIMITED TO, MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.	IN NO EVENT WILL MACROMEDIA BE
	LIABLE TO YOU FOR ANY CONSEQUENTIAL, INDIRECT OR INCIDENTAL DAMAGES ARISING OUT OF YOUR
	USE OR INABILITY TO USE THIS CODE.
*/


#define INITGUID 1

#include "ScreenXtra.h"
#include <CGDirectDisplay.h>
#include "MachoGlue.h"
#include "MyMacApplication.h"

#include <string.h>
#include <stdlib.h>
#include "xclassver.h"
#include "moatry.h"

#include "driservc.h"
#include "drivalue.h"

#include "mmivalue.h"
#include "mmillist.h"
#include "mmiplist.h"
#include "mmidate.h"
#include "mmiclr.h"
#include "mmiservc.h"

#include <displays.h>
#include <folders.h>
#include <stat.h>
#include <Resources.h>
#include <MacWindows.h>
#include <carbon.h>

#define strechtopage 1

/*******************************************************************************
 * SCRIPTING XTRA MESSAGE TABLE DESCRIPTION.
 *
 * The general format is:
 * xtra <nameOfXtra>
 * new object me [ args ... ]
 * <otherHandlerDefintions>
 * --
 * The first line must give the name of the Scripting xtra.
 * The remaining lines give the names of the handlers that this xtra implements
 * along with the required types of the arguments for each handler.
 * 
 * -- Pre-defined handler new 
 * The new handler will be called when a child object is created, 
 * the first argument is always the child object and you defined any remaining arguments.
 * The new handler is the place to initialize any memory required by the child object.
 * 
 * -- Simple Handler Definitions
 * Each handler definition line is format like this:
 * <handlerName> <argType1> <argName1>, <argType2> <argName2> ...
 * The first word is the handler name. Following this are types description for
 * the argument to the handler, each separated by a comma.
 * The argument name <argName>, may be omited.
 * Permited argument types are: 
 * 	integer 
 * 	float
 * 	string
 * 	symbol
 * 	object
 * 	any
 * 	*
 * For integer, float, string, symbol, and object, the type of the argument must 
 * match. The type any means allow any type. The asterisk (*) means any number and 
 * any type of arguments.
 * 
 * The first argument is the child object and is always declared to be of type object.
 * 
 * -- Global Handlers
 * An asterisk (*) preceeding the handler name signifies a global handler.
 * This handler is at the global scope level and can be called from any
 * movie.
 * 
 * -- Xtra level handlers
 * A plus (+) preceeding the handler name signifies an Xtra level handler. 
 * This kind of handler can be called directly from the Xtra reference,
 * without creating a child object.
 * 
 * The enumerated list that follows must correspond directly with the msgTable 
 * (i.e. they must be in the same order).
 * 
 *******************************************************************************/ 
 
 /* This is the list of handlers for the xtra. The versionInfo string is combined
 /*	with the msgTable string in the register method to create a single string that
 /* used when registering the xtra as a scripting xtra. */

static char versionInfo[] = "xtra ScreenXtra -- version %s.%s.%s\n";
static char msgTable[] = { 
	"new object me\n"  /* standard first handler entry in all message tables */
	"ChangeMode object me,integer width,integer height,integer depth -- change screen mode \n"
	"GetErrorText object me,integer errorcode -- returns text of error \n"

	};


enum 
{
	m_new = 0, /* standard first entry */
	m_changemode,
	m_geterrortext,
	m_XXXX
};


/* ============================================================================= */
/* Xtra Glue Stuff */
/* ============================================================================= */

#define XTRA_VERSION_NUMBER XTRA_CLASS_VERSION

BEGIN_XTRA
	BEGIN_XTRA_DEFINES_CLASS(ScreenXtra, XTRA_CLASS_VERSION)
		CLASS_DEFINES_INTERFACE(ScreenXtra, IMoaRegister, XTRA_VERSION_NUMBER)
		CLASS_DEFINES_INTERFACE(ScreenXtra, IMoaMmXScript, XTRA_VERSION_NUMBER)
		/*
		 * ---> insert additional interface(s) -->
		 */ 
	END_XTRA_DEFINES_CLASS
END_XTRA


/* ============================================================================= */
/* Create/Destroy for class ScreenXtra */
/* ============================================================================= */


STDMETHODIMP_(MoaError) MoaCreate_ScreenXtra (ScreenXtra FAR * This)
{
moa_try
		
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmValue, (PPMoaVoid) &This->pValueInterface));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmUtils2, (PPMoaVoid) &This->pMoaUtils));

moa_catch
moa_catch_end
moa_try_end
}

STDMETHODIMP_(void) MoaDestroy_ScreenXtra(ScreenXtra FAR * This)
{
moa_try

	if (This->pValueInterface != NULL) 
		ThrowErr (This->pValueInterface->Release());
		
	if (This->pMoaUtils != NULL) 
		ThrowErr (This->pMoaUtils->Release());

moa_catch
moa_catch_end
moa_try_end_void
}


/* ============================================================================= */
/* Methods in ScreenXtra_IMoaRegister */
/* ============================================================================= */

/*****************************************************************************
 *  Data needed for Registering
 *	---------------------------
 *	Specific code needed to register different types of Xtras.  The skeleton
 *	source files include minimal implementations appropriate for each Xtra
 *	type.  Current necessary actions:
 *
 *	Scripting Xtra:				Add the Scripting Xtra Message Table
 *	Sprite Asset Xtra:			Nothing
 *	Tool Xtra:					Nothing
 *	Transition Asset Xtra		Nothing
 *
 *  ****optional: Register as Safe for Shockwave!
 *****************************************************************************/ 

STD_INTERFACE_CREATE_DESTROY(ScreenXtra, IMoaRegister)

BEGIN_DEFINE_CLASS_INTERFACE(ScreenXtra, IMoaRegister)
END_DEFINE_CLASS_INTERFACE

/* ----------------------------------------------------------------------------- */
STDMETHODIMP ScreenXtra_IMoaRegister::Register(
	PIMoaCache pCache, 
	PIMoaXtraEntryDict pXtraDict
)
{	
moa_try
	PIMoaRegistryEntryDict		pReg;
	MoaBool						bItsSafe = TRUE;
	char versionStr[256];
	PMoaVoid pMemStr = NULL;

	/* Register the lingo xtra */
	ThrowErr(pCache->AddRegistryEntry(pXtraDict, &CLSID_ScreenXtra, &IID_IMoaMmXScript, &pReg));

	/* Register the method table */
	sprintf(versionStr, versionInfo, VER_MAJORVERSION_STRING, VER_MINORVERSION_STRING, VER_BUGFIXVERSION_STRING);
	pMemStr = pObj->pCalloc->NRAlloc(sizeof(msgTable) + sizeof(versionStr));
	ThrowNull(pMemStr);

	strcpy((char *)pMemStr, versionStr);
	strcat((char *)pMemStr, msgTable);

	ThrowErr(pReg->Put(kMoaDrDictType_MessageTable, pMemStr, 0, kMoaDrDictKey_MessageTable));

	/* Mark xtra as safe for shockwave - but only if it IS safe ! */
	/*
	ThrowErr(pReg->Put(kMoaMmDictType_SafeForShockwave, &bItsSafe, sizeof(bItsSafe), kMoaMmDictKey_SafeForShockwave));
	*/

moa_catch
moa_catch_end
	if (pMemStr)
		pObj->pCalloc->NRFree(pMemStr);
moa_try_end
}



/* ============================================================================= */
/*  Methods in ScreenXtra_IMoaMmXScript */
/* ============================================================================= */

BEGIN_DEFINE_CLASS_INTERFACE(ScreenXtra, IMoaMmXScript)
END_DEFINE_CLASS_INTERFACE

//******************************************************************************
ScreenXtra_IMoaMmXScript::ScreenXtra_IMoaMmXScript(MoaError FAR * pError)
//------------------------------------------------------------------------------
{
	*pError = kMoaErr_NoErr;
}	

//******************************************************************************
ScreenXtra_IMoaMmXScript::~ScreenXtra_IMoaMmXScript()
//------------------------------------------------------------------------------
{
}


/* ----------------------------------------------------------------------------- */
STDMETHODIMP ScreenXtra_IMoaMmXScript::Call (PMoaDrCallInfo callPtr)
{
moa_try
	switch	( callPtr->methodSelector ) 
	{
		case m_new:
			{
			/* Setup any instance vars for you Xtra here. new() is
			called via Lingo when creating a new instance. */
			/*
			 * --> insert additional code -->
		 	 */
			}
			break;
					

		case m_changemode:
			ThrowErr(XChangeMode(callPtr));
			break;

		case m_geterrortext:
			ThrowErr(XGetErrorText(callPtr));
			break;

	}
moa_catch
moa_catch_end
moa_try_end
}


/*****************************************************************************
 *  Private Methods
 *  -------------------
 *  Implementation of Private Class Methods
 /*
 *  This is the actual code for the defined methods. They are defined as 
 *  functions, and called from the switch statement in Call().
 *
 ****************************************************************************/

DMDisplayModeListEntryRec mrec;
pascal void MyDMDisplayModeListIteratorCallback ( void *userData, DMListIndexType itemIndex, DMDisplayModeListEntryPtr displaymodeInfo)
{

mrec=(*displaymodeInfo);

}

int switchres(int w,int h,int d)
{
	int	reterror=0;
#if TARGET_CARBON
#define kMaxDisplays	16	

	OSErr	err;
	CGDirectDisplayID display[kMaxDisplays];
	CGDisplayCount numDisplays;
	CGDisplayErr disperr;
	CFDictionaryRef mode;
	boolean_t exactMatch;
	CFBundleRef 		appBundle;
		
	typedef CGDisplayErr (*getdisplaylistptr)(CGDisplayCount maxDisplays,CGDirectDisplayID * activeDspys,CGDisplayCount * dspyCnt);
	typedef CFDictionaryRef (*MYDisplayBestModeForParametersPtr)(CGDirectDisplayID display,size_t bitsPerPixel,size_t width,size_t height,boolean_t * exactMatch);
	typedef CGDisplayErr (*MYDisplaySwitchToModePtr)(CGDirectDisplayID display, CFDictionaryRef mode);
	
	getdisplaylistptr getdisplaylist = nil;
	MYDisplayBestModeForParametersPtr TheDisplayBestModeForParameters=nil;
	MYDisplaySwitchToModePtr TheDisplaySwitchToMode=nil;
		
	err = LoadFrameworkBundle(CFSTR("ApplicationServices.framework"), &appBundle);
	getdisplaylist = (getdisplaylistptr) CFBundleGetFunctionPointerForName(appBundle,CFSTR("CGGetActiveDisplayList"));
	TheDisplayBestModeForParameters = (MYDisplayBestModeForParametersPtr) CFBundleGetFunctionPointerForName(appBundle,CFSTR("CGDisplayBestModeForParameters"));
	TheDisplaySwitchToMode = (MYDisplaySwitchToModePtr) CFBundleGetFunctionPointerForName(appBundle,CFSTR("CGDisplaySwitchToMode"));

	disperr = getdisplaylist(kMaxDisplays,display,&numDisplays);
	mode = TheDisplayBestModeForParameters(display[0],d,w,h,&exactMatch);
	disperr = TheDisplaySwitchToMode(display[0], mode);
		
	if (disperr==kCGErrorSuccess)
		{
		if(exactMatch)
			reterror=0;//extact match osx/classic
		else
			reterror=1;//changed osx
		}
	else
		reterror=2;//no change osx
		
#else
		DMDisplayModeListIteratorUPP MyDMDisplayModeListIteratorUPP; 
		Handle	dispstate;
		DMListIndexType cnt;
		DMListType plist;
		DisplayIDType	id;
		GDHandle md;
		unsigned long dp;
		Rect mr;
		OSErr	err;

		dp=d;
		md=DMGetFirstScreenDevice (true);
		err=DMBeginConfigureDisplays (&dispstate);
		MyDMDisplayModeListIteratorUPP = NewDMDisplayModeListIteratorUPP(&MyDMDisplayModeListIteratorCallback); 
 
		SetRect(&mr,0,0,w,h);
		reterror=5;//no matxh classic

		DMGetDisplayIDByGDevice (md,&id,true); 
		DMNewDisplayModeList (id,kDMModeListPreferSafeModesMask,0,&cnt,&plist);

		for(int nn=0;nn<=cnt;nn++)
			{

			DMGetIndexedDisplayModeFromList (plist,nn,0,MyDMDisplayModeListIteratorUPP,nil);

			if(((*mrec.displayModeResolutionInfo).csHorizontalPixels==mr.right) && ((*mrec.displayModeResolutionInfo).csVerticalLines==mr.bottom))
				{
				if ((mrec.displayModeFlags & 1)==0)
						{
						err=DMSetDisplayMode (md,(*mrec.displayModeResolutionInfo).csDisplayModeID,&dp,NULL,dispstate);
						if (err!=noErr)
							{
							reterror=3;//cant change classic
							}
						else
							{
							err=HasDepth (md,d,1,1);
 							if (err)
								reterror=0;//exact classic osx
							else
								reterror=4;//changed classic
								//err=DMEndConfigureDisplays (dispstate);
							goto exitok;
							}
						}
				}
			}
		
exitok:	
		err=DMEndConfigureDisplays (dispstate);


#endif

	return(reterror);

}


MoaError ScreenXtra_IMoaMmXScript::XChangeMode(PMoaDrCallInfo pCall)
{
MoaError	err = kMoaErr_NoErr;
MoaMmValue	value;
MoaLong		xres,yres,dp;
int			reterror;

moa_try

	pciGetArgByIndex( pCall, 2, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger(&value, &xres));

	pciGetArgByIndex( pCall, 3, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger(&value, &yres));

	pciGetArgByIndex( pCall, 4, &value );
	ThrowErr(pObj->pValueInterface->ValueToInteger(&value, &dp));


	reterror=switchres(xres,yres,dp);
	ThrowErr(pObj->pValueInterface->IntegerToValue(reterror,&pCall->resultValue));

	return kMoaErr_NoErr;
	
moa_catch
moa_catch_end	
moa_try_end	
}

MoaError ScreenXtra_IMoaMmXScript::XGetErrorText(PMoaDrCallInfo pCall)
{
moa_try
	

	MoaError		err = kMoaErr_NoErr;
	long			error;
	MoaMmValue		value = {0};
	char			*errorpath[2048];

	pciGetArgByIndex( pCall, 2, &value );
	err=pObj->pValueInterface->ValueToInteger( &value, &error);

	switch	(error) 
	{
		case 0:
			sprintf((char*)errorpath,"%s","Ok");
			break;

		case 1:
			sprintf((char*)errorpath,"%s","Changed not exact match OSX");
			break;

		case 2:
			sprintf((char*)errorpath,"%s","No change OSX");
			break;

		case 3:
			break;
			sprintf((char*)errorpath,"%s","Cant change classic");

		case 4:
			sprintf((char*)errorpath,"%s","Changed not exact match classic");
			break;
			
		case 5:
			sprintf((char*)errorpath,"%s","No such resolution classic");
			break;
			
	}

exit_gracefully:

	ThrowErr(pObj->pValueInterface->StringToValue((char*)errorpath,&pCall->resultValue));

return kMoaErr_NoErr;
moa_catch
moa_catch_end	
moa_try_end	
}

